plot.mixreg <- local({

plotmixregInfo <- new.env()

getVnms <- function(x) {
    vnms   <- all.vars(x$formula)
    nmy    <- vnms[1]
    noPred <- length(vnms) == 1
    if(noPred) {
        vnms <- c(vnms,"index")
        x$data[["index"]] <- 1:length(x$data[[nmy]])
    }
    if(length(vnms)==2) {
        nmx <- vnms[2]
        ok  <- ncol(as.matrix(x$data[[nmx]])) == 1
    } else {
        ok <- FALSE
    }
    if(!ok) {
        stop("This plot method can only be used for models with at most one predictor.\n")
    }
    list(x=x,noPred=noPred,names=c(nmx=nmx,nmy=nmy))
}

plot.mixregDual <- function(m1, m2, cMeth=c("none","distance","prob"),
                            ptcol=ptcol,col=NULL,col2=NULL,lty=NULL,
                            lty2=NULL,warn=TRUE,modnames=NULL,
                            legPos="topleft",...) {
    if(!(inherits(m1,"mixreg") & inherits(m2,"mixreg"))) {
        stop("Arguments \"m1\" and \"m2\" must both be of class \"mixreg\".\n")
    }
    plot.mixreg(m1,legPos=NULL,cMeth=cMeth,ptcol=ptcol,col=col,lty=lty,...)
    plot.mixreg(m2, legPos=legPos, add=TRUE, warn=warn, cMeth="none",
                modnames=modnames,col=col2,lty=lty2)
}

function(x,y,cMeth=c("none","distance","prob"),ptcol="black",
                     col=NULL,col2=NULL,lty=NULL,lty2=NULL,add=FALSE,
                     warn=TRUE,modnames=NULL,legPos="topleft",...) {
chknmx <- getVnms(x)
x      <- chknmx$x
if(!missing(y)) {
    if(inherits(y,"mixreg")) {
        chknmy <- getVnms(y)
        y      <- chknmy$x
        if(warn) {
            oldDat <- as.data.frame(x$data)[,chknmx$names]
            newDat <- as.data.frame(y$data)[,chknmy$names]
            if(!isTRUE(all.equal(newDat,oldDat))) {
                whinge <- paste0("Data for the plot being added appear to differ\n",
                                 "  from the data used to create the existing plot.\n")
                warning(whinge)
            }
        }
        plot.mixregDual(m1=x,m2=y,cMeth=cMeth,ptcol=ptcol,col=col,col2=col2,
                        lty=lty,lty2=lty2,warn=warn,modnames=modnames,
                        legPos=legPos,...)
        return(invisible())
    } else {
        stop("If \"y\" is supplied it must be of class \"mixreg\".\n")
    }
}
if(add) {
    fname <- as.character(sys.call(-1))[1]
    if(is.na(fname) | fname != "plot.mixregDual") {
        whinge <- paste0("When \"add\" is TRUE, plot.mixreg() must be called\n",
                         "  by plot.mixregDual.\n")
        stop(whinge)
    }
}

#
# Method to plot the result of fitting a mixture of regressions
# Model.  Plots the data and fitted regression lines.
#
# If the number of components is one, we don't need a legend.
if(missing(legPos) & nrow(x$parmat)==1) legPos <- NULL



cMeth <- match.arg(cMeth)

x$data <- as.data.frame(x$data)
K    <- nrow(x$parmat)
nmx  <- chknmx$names["nmx"]
nmy  <- chknmx$names["nmy"]
xr   <- range(x$data[,nmx])
yr   <- range(x$data[,nmy])
xlim <- list(...)$xlim
if(is.null(xlim)) xlim <- xr
if(chknmx$noPred) {
    xm <- matrix(c(1,1),ncol=1)
    jays <- 1
} else {
    xm   <- if(x$intercept) cbind(1,xlim) else matrix(xlim,ncol=1)
    jays <- if(x$intercept) 1:2 else 1
}
yh   <- xm%*%t(x$parmat[,jays,drop=FALSE])

# Colours and line types depend on callNum; ylim, xlab, ylab,
# pch and cex only used when callNum is 1.
callNum <- plotmixregInfo[["callNum"]]
cols <- col
if(is.null(cols)) {
    cols <- 1:K
} else {
    cols <- rep(cols,length=K)
}
ltys <- lty
if(is.null(ltys)) ltys <- 1
ltys <- rep(ltys,length=K)
if(!add) {
    ylim <- list(...)$ylim
    if(is.null(ylim)) ylim <- range(yr,yh)
    xlab <- list(...)$xlab
    if(is.null(xlab)) xlab <- nmx
    ylab <- list(...)$ylab
    if(is.null(ylab)) ylab <- nmy
    pch  <- list(...)$pch
    if(is.null(pch)) pch <- 1
    cex  <- list(...)$cex
    if(is.null(cex)) cex <- 1
} 

# Classify the points into groups.
if(cMeth=="none") {
    gps <- NULL
} else {
    mm  <- model.matrix(x$formula,data=x$data)
    nmy <- as.character(x$formula[[2]])
    if(cMeth=="distance") {
        xx  <- if(x$intercept) mm[,2] else mm[,1]
        yy  <- x$data[[nmy]]
        dM  <- distMat(xx,yy,x$theta)
        gps <- factor(apply(dM,1,which.min))
    } else if(cMeth=="prob") {
        xxx <- gfun(mm,x$data[[nmy]],x$theta)
        gps <- factor(apply(xxx$gamma,1,which.max))
    } else {
         stop(paste("Classification method",cMeth,"not recognized.\n"))
    }
}

if(!add) {
    plot(0,0,type='n',xlim=xlim,ylim=ylim,xlab=xlab,ylab=ylab,
         main=list(...)$main)
}
kull <- if(cMeth=="none") ptcol else cols[gps]
if(!add) points(x$data[,nmx],x$data[,nmy], col=kull, pch=pch, cex=cex)
for(j in 1:K) {
    lines(xlim,yh[,j],col=cols[j],lty=ltys[j])
}
if(!is.null(legPos)) {
    if(add) {
# Life gets complicated.
        Info  <- plotmixregInfo[["Info"]]
        ltys1 <- Info[["ltys"]]
        cols1 <- Info[["cols"]]
        if(is.null(modnames)) modnames <- c("Model 1","Model 2")
        leg <- c(paste0(modnames[1], ", Component ",1:K),
                 paste0(modnames[2], ", Component ",1:K))
        ltys <- c(ltys1,ltys)
        cols <- c(cols1,cols)
    } else {
        leg <- paste("Component",1:K)
    }
    legend(legPos,lty=ltys,col=cols,legend=leg,bty="n")
}

if(cMeth=="none") {
    rslt <- NULL
} else {
    rslt <- cbind(x$data,groups=gps)
    attr(rslt,"cMeth") <- cMeth
}
if(!add) {
    assign("Info",list(ltys=ltys,cols=cols),envir=plotmixregInfo)
}
invisible(rslt)
}
})
