\name{plot.mixreg}
\alias{plot.mixreg}
\title{
   Plot fitted mixtures of regressions.
}
\description{
   Plots the fitted regression lines for one or two mixtures of
   regression models as fitted by \code{mixreg()} (or possibly by
   \code{visualFit()}).
}
\usage{
   \method{plot}{mixreg}(x, y, cMeth=c("none","distance","prob"),
                         ptcol="black",col=NULL,col2=NULL,lty=NULL,lty2=NULL,
                         add=FALSE, warn=TRUE, modnames=NULL, legPos = "topleft", ...)
}
\arguments{
  \item{x}{
  An object of class \code{"mixreg"} as returned by \code{\link{mixreg}()}
  or possibly by \code{\link{visualFit}()}.
}
  \item{y}{
  If supplied it must be an object of class \code{"mixreg"}.
}
  \item{cMeth}{
  Text string specifying the \dQuote{classification method}.
  If \code{cMeth} is \code{"none"} no classification is done. If
  \code{cMeth} is \code{"distance"} then points are classified
  as \dQuote{belonging} to a component of the mixture according
  to which of the regression lines they are nearest to, in terms
  of Euclidean distance.  If \code{cMeth} is \code{"prob"} then
  points are classified as \dQuote{belonging} to a component
  of the mixture according to which of the probabilities
  \eqn{\gamma_{ij}}{gamma[i,j]} of their being associated with
  that component is largest.  If the points are classified then
  they are plotted in colours corresponding to that classification.
}
  \item{ptcol}{
  Scalar specfying the colour in which the data points are plotted.
  Ignored unless \code{cMeth="none"}.  (In other cases the data points
  are plotted in colours corresponding to their classification.)
}
  \item{col}{
  Vector specifying the colours in which the fitted lines, corresponding
  to each of the components, should be plotted.  Defaults to \code{1:K}
  where \code{K} is the number of components.  These are also the colours
  in which the data points corresponding to each of the components are
  plotted when \code{cMeth} differs from \code{"none"}.
}
  \item{col2}{
  Like \code{col} except that it specifies the colours used to plot
  the second model, specified by \code{y}.  Ignored if \code{y}
  is missing.  Not used for point colours.
}
  \item{lty}{
  Vector specifying the line types used for plotting the fitted lines
  corresponding to each of the components.  Defaults to \code{1} (i.e.
  all lines are solid).
}
  \item{lty2}{
  Like \code{lty} except that it specifies the line types used to plot
  the second model, specified by \code{y}.  Ignored if \code{y}
  is missing.
}
  \item{add}{
  For internal use only.
}
  \item{warn}{
  Logical scalar; should a warning be issued if the data for the
  plot being added (those corresponding to argument \code{y})
  differ from those for the existing plot?  Ignored if \code{y}
  is missing.
}
  \item{modnames}{
  Character vector, of length 2, of \dQuote{model names}.  Used by
  \code{legend()} when adding the plot of the second model (specified
  by \code{y}) to the initial plot.  Ignored if \code{y} is missing,
  and of course is also ignored if \code{legPos} is \code{NULL}. The
  first entry is the name of the model corresponding to \code{x}
  and the second is the name corresponding to \code{y}.  Defaults to
  \code{c("Model 1","Model 2")}.
}
  \item{legPos}{
  A list with entries \code{x} and \code{y}, or a text string,
  specifying the placement of the legend.  See \code{\link{legend}()}
  for details.  The plotting of a legend may be suppressed by
  setting \code{legPos=NULL}.  Only one legend is plotted even
  if two models are specified; which \dQuote{pieces} correspond
  to which model is indicated by \code{modnames}.
}
  \item{\dots}{
  Optional extra arguments for \code{plot()}, \code{points()}
  and \code{lines()}.  These may include \code{xlim}, \code{ylim},
  \code{xlab}, \code{ylab}, \code{pch}, and \code{main}.  Note that
  \code{\dots} is used only when plotting the model specified
  by \code{x}.
}
}
\section{Details about superimposition of plots}{
Superimposing a plot of one fitted mixture of regressions model
upon that of another such model really makes sense only if the
same underlying data set is involved.  The code checks whether the
data sets appear to be the same, and if not, issues a warning when
\code{warn} is \code{TRUE}.

Note that only the data points corresponding to the first model
(i.e. that corresponding to argument \code{x}) are plotted.  The
resulting graphic would be too cluttered otherwise, and anyway,
as was said initially, the plot doesn't really make sense if
different data sets are involved.
}

\value{
  If both \code{x} and \code{y} models are both specified, or
  if \code{cMeth} is \code{"none"}, then no value is returned.
  Otherwise the value is the \code{data} component of the
  \code{x} argument of this function, augmented by an extra
  column \code{groups}.  This column is a factor that specifies
  the component to which each point has been assigned.  This data
  frame also has an attribute \code{"cMeth"}, the value of the
  \code{cMeth} argument.
}
\author{Rolf Turner
  \email{r.turner@auckland.ac.nz}
}
\seealso{
\code{\link{plot.cband}()}, \code{\link{plot.mixresid}()},
\code{\link{qqMix}()},
}
\examples{
    thStrt <- list(
                   list(beta=c(26.07,48808),sigsq=1.1573,lambda=0.33333333),
                   list(beta=c(23.48,32387),sigsq=1.8730,lambda=0.33333333),
                   list(beta=c(-0.0597,20760),sigsq=0.2478,lambda=0.33333333)
                 )
    kfit  <- mixreg(y ~ x,ncomp=3,data=kilnAoneOut,thetaStart=thStrt)
    plot(kfit,pch=8,col=c("red","green","blue","black"))
    plot(kfit,pch=8,col=c("green","blue","black"),cMeth="d")
    plot(kfit,pch=8,col=c("green","blue","black"),cMeth="p")
    afit <- mixreg(plntsInf ~ aphRel,data=aphids,ncomp=2)
    plot(afit,cMeth="p",col=c("blue","red"),pch=20)
    # Separates the points into two groups incredibly clearly!
    npfit <- mixreg(plntsInf ~ 1, data=aphids,ncomp=2)
    plot(npfit,cMeth="p",col=c("blue","red"),pch=20)
    \dontrun{
        vfit <- visualFit(plntsInf ~ aphRel,data=aphids,ncomp=2)
        afit <- mixreg(plntsInf ~ aphRel,data=aphids,thetaStart=vfit$theat)
        plot(vfit,afit,col=c("black","red"),col2=c("blue","green"),
             modnames=c("Visual","Analytic"))
        bfit <- mixreg(plntsInf ~ aphRel,data=aphids)
        plot(afit,bfit,col=c("black","red"),col2=c("blue","green"),
             modnames=c("Visual start","Random start"))
        # Results from visual start and random start look identical.
    }
    fit.real <- mixreg(plntsInf ~ aphRel,data=aphids,ncomp=2)
    simDat   <- rmixreg(fit.real)
    fit.sim  <- mixreg(plntsInf ~ aphRel,data=simDat,ncomp=2)
    plot(fit.real,fit.sim,col=c("black","red"),col2=c("blue","green"),
         lty=1:2,lty2=3:4,warn=FALSE,legPos="topleft",cMeth="p",pch=20,
         modnames=c("Real data","Simulated data"),
         main="Real data points plotted")
    # We set warn=FALSE since we are well aware that the data sets differ.
    plot(fit.sim,fit.real,col2=c("black","red"),col=c("blue","green"),
         lty2=1:2,lty=3:4,warn=FALSE,legPos="topleft",cMeth="p",pch=20,
         modnames=c("Simulated data","Real data"),
         main="Simuated data points plotted.")
}
\keyword{ hplot }
